/*
 * throttle - run a program under a pty, and throttle its output
 */

#include <stdio.h>
#ifdef __linux__
#include <pty.h>
#elif __FreeBSD__
#include <sys/types.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <libutil.h>
#else /* NetBSD */
#include <util.h>
#endif
#include <poll.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>

struct termios t_orig;

void cleanup() {
    (void)tcsetattr(0, TCSAFLUSH, &t_orig);
    exit(0);
}

/* Copy a character from file descriptor i to file descriptor o */

void copy(int i, int o) {
    char c;
    int r;
    r = read(i, &c, 1);
    if (r <= 0)
        cleanup();
    r = write(o, &c, 1);
    if (r <= 0)
        cleanup();
}

int main(int argc, char **argv) {
    int master;
    pid_t child = forkpty(&master, NULL, NULL, NULL);
    if (child == 0) {
        execvp(argv[1], argv + 1);
        exit(1);
    }

    struct termios t;
    (void)tcgetattr(0, &t);
    t_orig = t;
    cfmakeraw(&t);
    t.c_lflag &= ~ECHO;
    (void)tcsetattr(0, TCSAFLUSH, &t);

    struct pollfd pollfds[2];
    pollfds[0].fd = 0;
    pollfds[0].events = POLLIN;
    pollfds[1].fd = master;
    pollfds[1].events = POLLIN;

    for (;;) {
        int n = poll(pollfds, 2, -1);
        if (n < 0)
            break;
        if (pollfds[0].revents & POLLIN)
            copy(0, master);
        if (pollfds[1].revents & POLLIN) {
            copy(master, 1);
            poll(NULL, 0, 5);
        }
    }
}
