/*
 * throttlee - an example program to throttle
 *
 * Prints lines of the ASCII chars 0 through z in an infinite loop,
 * with a timer firing once every 15 ms (nominal) to cause interupted
 * system calls.
 */

#include <sys/time.h>

#include <errno.h>
#include <stdlib.h>
#include <signal.h>
#include <memory.h>
#include <unistd.h>

void timer_tick(int signo) {
    /* do nothing */ 
}

void put(int c) {
    int r;
retry:
    r = write(0, &c, 1);
    if (r < 0 && errno == EINTR)
        goto retry;
    if (r <= 0)
        exit(1);
}

int main(int argc, char **argv) {
    struct itimerval val;
    struct sigaction saction;
    int i;
    
    val.it_interval.tv_sec = 0;
    val.it_interval.tv_usec = 15000;
    val.it_value.tv_sec = 0;
    val.it_value.tv_usec = 15000;
    
    memset(&saction, 0, sizeof(saction));
    saction.sa_handler = timer_tick;
    
    sigaction(SIGALRM, &saction, NULL);
    
    setitimer(ITIMER_REAL, &val, NULL);

    for (;;) {
        for (i = '0'; i <= 'z'; i++) {
            put(i);
        }
        put('\n');
    }
}
